#include<MT/truncatedGaussian.h>
#include<MT/plot.h>
#include<MT/util.h>

void plotTruncationLine(const arr& c,const arr& coff);

void testTruncGauss(){
  uint N=MT::getParameter<uint>("dim",2);
  //random covariance and mean
  arr A,M(N,N),a(N),cdir(N),coff(N);
  rndUni(M,.1,1.);
  A=~M*M;
  rndUni(a,-1.,1.);
  //a(0)=.0;  a(1)=0.;  A<<"[1. 1.; 1. 2.]";

  //random constraint
  rndUni(coff,-1.,1.);
  rndUni(cdir,-1.,1.);
  //coff(0)=1.;  coff(1)=0.;  cdir(0)=1.;  cdir(1)=0.;
  
  //plot problem
  plotClear();
  plotCovariance(a,A);
  plotTruncationLine(cdir,coff);

  //analytical solution
  arr b,B;
  b=a; B=A;
  TruncateGaussian(b,B,cdir,scalarProduct(cdir,coff));
  plotCovariance(b,B);

  //sampling solution
  b=a; B=A;
  arr data;
  TruncateGaussianBySampling(b,B,cdir,scalarProduct(cdir,coff),MT::getParameter<uint>("samples",3000),&data);
  plotPoints(data);
  plotCovariance(b,B);
  plot();
}

void plotTruncationLine(const arr& c,const arr& coff){
  arr line(2,2);
  line(0,0)=coff(0);     line(0,1)=coff(1);
  line(1,0)=line(0,0)+c(0); line(1,1)=line(0,1)+c(1);
  plotLine(line);
}

int main(int argc, char *argv[]){
  cout <<"USAGE: ./x.exe -seed <integer> -dim <integer> -samples <integer>\n" <<endl;
  
  MT::initCmdLine(argc,argv);

  for(;;) testTruncGauss();
  
  return 0;
}

